#include "ISO9660FileSystem.h"
#include "VirtualCD.h"

#include <stdio.h>
#include <string.h>

using namespace MonAPI;

bool read(FileSystem* iso, const char* filePath)
{
    File* file = iso->Open(filePath, 0);

    if (file == NULL)
    {
        printf("read:file not found\n");
        return false;
    }

    char* buffer = new char[file->GetSize()];
    file->Seek(0, SEEK_SET);
    file->Read(buffer, file->GetSize());

    for (dword i = 0; i < file->GetSize(); i++)
    {
        printf("%c", buffer[i]);
    }

    delete [] buffer;

    iso->Close(file);

    return true;
}

bool cp(FileSystem* iso, const char* filePath, const char* outPath)
{
    File* file = iso->Open(filePath, 0);

    if (file == NULL)
    {
        printf("cp:file not found\n");
        return false;
    }

    char* buffer = new char[file->GetSize()];
    file->Seek(0, SEEK_SET);
    file->Read(buffer, file->GetSize());

    FILE* out = fopen(outPath, "wb");
    if (out == NULL)
    {
        iso->Close(file);
        delete [] buffer;
        return false;
    }

    fwrite(buffer, sizeof(byte), file->GetSize(), out);
    fclose(out);
    iso->Close(file);
    delete [] buffer;
    return true;
}


int main(int argc, char *argv[])
{
    if (argc < 4)
    {
        printf("usage: ISO9660.exe isoImage comand [option|file|directory|\n");
        printf("command: read, ls, cp\n");
        return 1;
    }

    const char* isoImage = argv[1];
    const char* command  = argv[2];
    const char* arg1     = argv[3];
    const char* arg2     = argc >= 4 ? argv[4] : NULL;

    FILE* fp = fopen(isoImage, "rb");

    if (fp == NULL)
    {
        printf("file %s can not read\n", isoImage);
        return 1;
    }

    IStorageDevice* cd = new VirtualCD(fp);
    ISO9660FileSystem* fs = new ISO9660FileSystem(cd);

    if (!fs->Initialize())
    {
        printf("Initialize Error = %d\n", fs->GetLastError());
        delete fs;
        delete cd;
        fclose(fp);
        return 1;
    }

    if (strcmp(command, "read") == 0)
    {
        read(fs, arg1);
    }
    else if (strcmp(command, "cp") == 0)
    {
        cp(fs, arg1, arg2);
    }
    else if (strcmp(command, "ls") == 0 && strcmp(arg1, "-l") == 0)
    {
        _A<FileSystemEntry*> files = fs->GetFileSystemEntries(arg2);

        char* month[] = {"", "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

        FOREACH (FileSystemEntry*, file, files)
        {
            FileDate* date = file->GetCreateDate();

            printf("%d %s %d %d %s%s\n"
                   , file->IsDirectory() ? 0 : file->GetSize()
                   , month[date->GetMonth()]
                   , date->GetDay()
                   , date->GetYear()
                   , (const char*)file->GetName()
                   , file->IsDirectory() ? "/" : "");
        }
        END_FOREACH
    }
    else if (strcmp(command, "ls") == 0)
    {
        _A<FileSystemEntry*> files = fs->GetFileSystemEntries(arg1);

        FOREACH (FileSystemEntry*, file, files)
        {
            printf("%s\n", (const char*)file->GetName());
        }
        END_FOREACH
    }
    else
    {
        printf("command %s not supported\n", command);
    }

    delete fs;
    delete cd;
    fclose(fp);

    return 0;
}
